#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/kobject.h>
#include <linux/notifier.h>
#include <linux/module.h>

/*
 * Generate uevent when link state change
 */
static int ifevent_handler(struct notifier_block *this,
                           unsigned long event,
                           void *ptr)
{
	struct net_device *dev = netdev_notifier_info_to_dev(ptr);

	switch (event) {
	case NETDEV_CHANGE:
		kobject_uevent(&dev->dev.kobj, KOBJ_CHANGE);
		break;
	case NETDEV_FATAL_ERROR:
		kobject_uevent(&dev->dev.kobj, KOBJ_OFFLINE);
		break;
	default:
		break;
	}

	return NOTIFY_DONE;
}

static struct notifier_block ifevent_notifier = {
	.notifier_call = ifevent_handler,
};

static int __init ifevent_init(void)
{
	return register_netdevice_notifier(&ifevent_notifier);
}

static void __exit ifevent_exit(void)
{
	unregister_netdevice_notifier(&ifevent_notifier);
	return;
}

module_init(ifevent_init);
module_exit(ifevent_exit);

MODULE_AUTHOR("Sophia Luo <sophial@netapp.com>");
MODULE_DESCRIPTION("Network Interface Link State");
MODULE_LICENSE("GPL");
